# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.networks.tokens import top_holder_get_params
from .....types.onchain.networks.tokens.top_holder_get_response import TopHolderGetResponse

__all__ = ["TopHoldersResource", "AsyncTopHoldersResource"]


class TopHoldersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TopHoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TopHoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopHoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TopHoldersResourceWithStreamingResponse(self)

    def get(
        self,
        address: str,
        *,
        network: str,
        holders: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TopHolderGetResponse:
        """
        This endpoint allows you to **query top token holders based on the provided
        token contract address on a network**

        Args:
          holders: number of top token holders to return, you may use any integer or `max` Default
              value: 10

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not address:
            raise ValueError(f"Expected a non-empty value for `address` but received {address!r}")
        return self._get(
            f"/onchain/networks/{network}/tokens/{address}/top_holders",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"holders": holders}, top_holder_get_params.TopHolderGetParams),
            ),
            cast_to=TopHolderGetResponse,
        )


class AsyncTopHoldersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTopHoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTopHoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopHoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTopHoldersResourceWithStreamingResponse(self)

    async def get(
        self,
        address: str,
        *,
        network: str,
        holders: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TopHolderGetResponse:
        """
        This endpoint allows you to **query top token holders based on the provided
        token contract address on a network**

        Args:
          holders: number of top token holders to return, you may use any integer or `max` Default
              value: 10

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not address:
            raise ValueError(f"Expected a non-empty value for `address` but received {address!r}")
        return await self._get(
            f"/onchain/networks/{network}/tokens/{address}/top_holders",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"holders": holders}, top_holder_get_params.TopHolderGetParams),
            ),
            cast_to=TopHolderGetResponse,
        )


class TopHoldersResourceWithRawResponse:
    def __init__(self, top_holders: TopHoldersResource) -> None:
        self._top_holders = top_holders

        self.get = to_raw_response_wrapper(
            top_holders.get,
        )


class AsyncTopHoldersResourceWithRawResponse:
    def __init__(self, top_holders: AsyncTopHoldersResource) -> None:
        self._top_holders = top_holders

        self.get = async_to_raw_response_wrapper(
            top_holders.get,
        )


class TopHoldersResourceWithStreamingResponse:
    def __init__(self, top_holders: TopHoldersResource) -> None:
        self._top_holders = top_holders

        self.get = to_streamed_response_wrapper(
            top_holders.get,
        )


class AsyncTopHoldersResourceWithStreamingResponse:
    def __init__(self, top_holders: AsyncTopHoldersResource) -> None:
        self._top_holders = top_holders

        self.get = async_to_streamed_response_wrapper(
            top_holders.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from coingecko_sdk import Coingecko, AsyncCoingecko
from coingecko_sdk.types.onchain.networks.pools import OhlcvGetTimeframeResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOhlcv:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_timeframe(self, client: Coingecko) -> None:
        ohlcv = client.onchain.networks.pools.ohlcv.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        )
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_timeframe_with_all_params(self, client: Coingecko) -> None:
        ohlcv = client.onchain.networks.pools.ohlcv.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
            token="token",
            aggregate="aggregate",
            before_timestamp=0,
            currency="usd",
            include_empty_intervals=True,
            limit=0,
        )
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_timeframe(self, client: Coingecko) -> None:
        response = client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ohlcv = response.parse()
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_timeframe(self, client: Coingecko) -> None:
        with client.onchain.networks.pools.ohlcv.with_streaming_response.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ohlcv = response.parse()
            assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_timeframe(self, client: Coingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network` but received ''"):
            client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
                timeframe="day",
                network="",
                pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pool_address` but received ''"):
            client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
                timeframe="day",
                network="eth",
                pool_address="",
            )


class TestAsyncOhlcv:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_timeframe(self, async_client: AsyncCoingecko) -> None:
        ohlcv = await async_client.onchain.networks.pools.ohlcv.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        )
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_timeframe_with_all_params(self, async_client: AsyncCoingecko) -> None:
        ohlcv = await async_client.onchain.networks.pools.ohlcv.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
            token="token",
            aggregate="aggregate",
            before_timestamp=0,
            currency="usd",
            include_empty_intervals=True,
            limit=0,
        )
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_timeframe(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ohlcv = await response.parse()
        assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_timeframe(self, async_client: AsyncCoingecko) -> None:
        async with async_client.onchain.networks.pools.ohlcv.with_streaming_response.get_timeframe(
            timeframe="day",
            network="eth",
            pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ohlcv = await response.parse()
            assert_matches_type(OhlcvGetTimeframeResponse, ohlcv, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_timeframe(self, async_client: AsyncCoingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network` but received ''"):
            await async_client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
                timeframe="day",
                network="",
                pool_address="0x06da0fd433c1a5d7a4faa01111c044910a184553",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pool_address` but received ''"):
            await async_client.onchain.networks.pools.ohlcv.with_raw_response.get_timeframe(
                timeframe="day",
                network="eth",
                pool_address="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["NewPoolGetParams"]


class NewPoolGetParams(TypedDict, total=False):
    include: str
    """
    attributes to include, comma-separated if more than one to include Available
    values: `base_token`, `quote_token`, `dex`, `network`
    """

    page: int
    """page through results Default value: 1"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.onchain.pools import trending_search_get_params
from ....types.onchain.pools.trending_search_get_response import TrendingSearchGetResponse

__all__ = ["TrendingSearchResource", "AsyncTrendingSearchResource"]


class TrendingSearchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TrendingSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TrendingSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrendingSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TrendingSearchResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        include: str | NotGiven = NOT_GIVEN,
        pools: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingSearchGetResponse:
        """
        This endpoint allows you to **query all the trending search pools across all
        networks on GeckoTerminal**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`

          pools: number of pools to return, maximum 10 Default value: 4

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/onchain/pools/trending_search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include": include,
                        "pools": pools,
                    },
                    trending_search_get_params.TrendingSearchGetParams,
                ),
            ),
            cast_to=TrendingSearchGetResponse,
        )


class AsyncTrendingSearchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTrendingSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrendingSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrendingSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTrendingSearchResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        include: str | NotGiven = NOT_GIVEN,
        pools: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingSearchGetResponse:
        """
        This endpoint allows you to **query all the trending search pools across all
        networks on GeckoTerminal**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`

          pools: number of pools to return, maximum 10 Default value: 4

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/onchain/pools/trending_search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include": include,
                        "pools": pools,
                    },
                    trending_search_get_params.TrendingSearchGetParams,
                ),
            ),
            cast_to=TrendingSearchGetResponse,
        )


class TrendingSearchResourceWithRawResponse:
    def __init__(self, trending_search: TrendingSearchResource) -> None:
        self._trending_search = trending_search

        self.get = to_raw_response_wrapper(
            trending_search.get,
        )


class AsyncTrendingSearchResourceWithRawResponse:
    def __init__(self, trending_search: AsyncTrendingSearchResource) -> None:
        self._trending_search = trending_search

        self.get = async_to_raw_response_wrapper(
            trending_search.get,
        )


class TrendingSearchResourceWithStreamingResponse:
    def __init__(self, trending_search: TrendingSearchResource) -> None:
        self._trending_search = trending_search

        self.get = to_streamed_response_wrapper(
            trending_search.get,
        )


class AsyncTrendingSearchResourceWithStreamingResponse:
    def __init__(self, trending_search: AsyncTrendingSearchResource) -> None:
        self._trending_search = trending_search

        self.get = async_to_streamed_response_wrapper(
            trending_search.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DexGetParams"]


class DexGetParams(TypedDict, total=False):
    page: int
    """page through results Default value: 1"""
